// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file SolverMatrix.h */

#ifndef AMDIS_SOLVERMATRIX_H
#define AMDIS_SOLVERMATRIX_H

#include "DOFMatrix.h"

namespace AMDiS {

  /**
   * \brief Helper class to provide complete matrix for ITL solvers and preconditioners
   */

  // General case undefined !
  template <typename Matrix>
  class SolverMatrix { };


  // DOFMatrix, i.e. ScalarProblem
  template <>
  class SolverMatrix<DOFMatrix>
  {
    public :

      SolverMatrix() : 
	matrix(0)
      {}

      void setMatrix(const DOFMatrix& A)
      {
	matrix= &A.getBaseMatrix();
      }

      const DOFMatrix::base_matrix_type& getMatrix() const 
      { 
	return *matrix; 
      }

    private:
      const DOFMatrix::base_matrix_type* matrix;
  };


  // VectorProblem
  template <>
  class SolverMatrix<Matrix<DOFMatrix*> >
  {
    void buildMatrix() const;
  public :
    SolverMatrix<Matrix<DOFMatrix* > >():
      originalMat(NULL)
    {}

    void setMatrix(const Matrix<DOFMatrix*>& A)
    {
      originalMat = &A;
      matrix.change_dim(0,0);
    }

    const DOFMatrix::base_matrix_type& getMatrix() const 
    { 
      if( num_rows(matrix) == 0)
        buildMatrix();
      return matrix; 
    }

    const Matrix<DOFMatrix*>* getOriginalMat() const
    {
      return originalMat;
    }
    
  private:      
    mutable DOFMatrix::base_matrix_type matrix;
    
    const Matrix<DOFMatrix*>* originalMat;

  };


} // namespace AMDiS

#endif // AMDIS_SOLVERMATRIX_H
