// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file PetscProblemStat.h */

#ifndef AMDIS_PETSC_PROBLEM_STAT_H
#define AMDIS_PETSC_PROBLEM_STAT_H

#include "AMDiS_fwd.h"
#include "Global.h"
#include "parallel/ParallelProblemStatBase.h"
#include "parallel/PetscSolver.h"
#include "parallel/PetscSolverFeti.h"
#include "parallel/PetscSolverGlobalMatrix.h"
#include "parallel/PetscSolverGlobalBlockMatrix.h"
#include "parallel/PetscSolverSchur.h"

namespace AMDiS {
  
  class PetscProblemStat : public ParallelProblemStatBase
  {
  public:
    PetscProblemStat(std::string nameStr, 
		     ProblemIterationInterface *problemIteration = NULL);

    PetscProblemStat(std::string nameStr,
		     PetscSolver *solver);

    ~PetscProblemStat()
    {
      if (deleteSolver)
	delete petscSolver;
    }

    void initialize(Flag initFlag,
		    ProblemStatSeq *adoptProblem = NULL,
		    Flag adoptFlag = INIT_NOTHING);

    void solve(AdaptInfo *adaptInfo,
	       bool createMatrixData = true,
	       bool storeMatrixData = false);

    PetscSolver* getPetscSolver()
    {
      return petscSolver;
    }

  protected:
    PetscSolver *petscSolver;

    bool deleteSolver;
  };

  typedef PetscProblemStat ParallelProblemStat;

#ifdef HAVE_PARALLEL_DOMAIN_AMDIS
  typedef PetscProblemStat ProblemStat;
#endif

} // namespace AMDiS

#endif
